@@ -103,10 +103,10 @@ static shell::numeric_t numeric_mode = shell::NO_NUMERIC;
 static long numeric_value = 0l;
 
 shell::Option::Option(char shortopt, const char *longopt, const char *value, const char *help) :
-OrderedObject()
+LinkedObject()
 {
     if(olast) {
-        olast->next = this;
+        olast->Next = this;
         olast = this;
     }
     else
@@ -290,7 +290,7 @@ void shell::set0(char *argv0)
         return;
 
     if(*argv0 != '/' && *argv0 != '\\' && argv0[1] != ':') {
-        fsys::getPrefix(prefix, sizeof(prefix));
+        fsys::prefix(prefix, sizeof(prefix));
         String::add(prefix, sizeof(prefix), "/");
         String::add(prefix, sizeof(prefix), argv0);
     }
@@ -318,7 +318,7 @@ void shell::set0(char *argv0)
 
 #ifdef  _MSWINDOWS_
     char *ext = strrchr(_argv0, '.');
-    if(case_eq(ext, ".exe") || case_eq(ext, ".com"))
+    if(eq_case(ext, ".exe") || eq_case(ext, ".com"))
         *ext = 0;
 #endif
 
@@ -930,9 +930,9 @@ static void pathfinder(const char *name, char *buf, size_t size)
     while(NULL != (element = String::token(path, &tokbuf, ";"))) {
         snprintf(buf, sizeof(buf), "%s\\%s", element, name);
         ext = strrchr(buf, '.');
-        if(!ext || (!case_eq(ext, ".exe") && !case_eq(ext, ".com")))
+        if(!ext || (!eq_case(ext, ".exe") && !eq_case(ext, ".com")))
             String::add(buf, size, ".exe");
-        if(fsys::isfile(buf))
+        if(fsys::is_file(buf))
             return;
     }
 
@@ -940,7 +940,7 @@ static void pathfinder(const char *name, char *buf, size_t size)
 
 tail:
     ext = strrchr(buf, '.');
-    if(!ext || (!case_eq(ext, ".exe") && !case_eq(ext, ".com")))
+    if(!ext || (!eq_case(ext, ".exe") && !eq_case(ext, ".com")))
         String::add(buf, size, ".exe");
 }
 
@@ -989,29 +989,29 @@ String shell::path(path_t id)
     case USER_DATA:
         if(GetEnvironmentVariable("APPDATA", buf, sizeof(buf))) {
             result = str(buf) + "\\" + _domain;
-            fsys::createDir(*result, 0700);
+            dir::create(*result, fsys::OWNER_PRIVATE);
         }
         break;
     case USER_CONFIG:
         if(GetEnvironmentVariable("USERPROFILE", buf, sizeof(buf))) {
             result = str(buf) + "\\Local Settings\\" + _domain;
-            fsys::createDir(*result, 0700);
+            dir::create(*result, fsys::OWNER_PRIVATE);
         }
         break;
     case USER_CACHE:
     case SERVICE_CACHE:
         if(GetEnvironmentVariable("TEMP", buf, sizeof(buf))) {
             result = str(buf) + "\\" + _domain;
-            fsys::createDir(*result, 0700);
+            dir::create(*result, fsys::OWNER_PRIVATE);
             break;
         }
         if(GetEnvironmentVariable("APPDATA", buf, sizeof(buf))) {
             result = str(buf) + "\\" + _domain;
-            fsys::createDir(*result, 0700);
+            dir::create(*result, fsys::OWNER_PRIVATE);
         }
         break;
     case SYSTEM_TEMP:
-        fsys::createDir("c:\\temp", 0777);
+        dir::create("c:\\temp", fsys::DIR_TEMPORARY);
         result ^= "c:\\temp";
         break;
     case PROGRAM_TEMP:
@@ -1042,7 +1042,7 @@ String shell::path(path_t id)
     return result;
 }
 
-const char *shell::getenv(const char *id, const char *value)
+const char *shell::env(const char *id, const char *value)
 {
     char buf[512];
     char path[255];
@@ -1360,6 +1360,7 @@ int shell::detach(const char *path, char **argv, char **envp, fd_t *stdio)
 
     pid = pi.hProcess;
     CloseHandle(pi.hThread);
+    err = 0;
 
 exit:
     if(ep)
@@ -1395,7 +1396,7 @@ char *shell::getpass(const char *prompt, char *buffer, size_t size)
 
 int shell::inkey(const char *prompt)
 {
-    if(prompt && fsys::istty(shell::input()))
+    if(prompt && fsys::is_tty(shell::input()))
         fputs(prompt, stdout);
     else
         return 0;
@@ -1414,7 +1415,7 @@ char *shell::getline(const char *prompt, char *buffer, size_t size)
 {
     unsigned pos = 0;
 
-    if(!fsys::istty(shell::input()))
+    if(!fsys::is_tty(shell::input()))
         return fgets(buffer, size, stdin);
 
     fputs(prompt, stdout);
@@ -1476,7 +1477,7 @@ char *shell::getline(const char *prompt, char *buffer, size_t size)
 {
     size_t pos = 0;
 
-    if(!fsys::istty(input()))
+    if(!fsys::is_tty(input()))
         return fgets(buffer, size, stdin);
 
     noecho(1);
@@ -1529,7 +1530,7 @@ char *shell::getpass(const char *prompt, char *buffer, size_t size)
 
 int shell::inkey(const char *prompt)
 {
-    if(!fsys::istty(1))
+    if(!fsys::is_tty(1))
         return 0;
 
     noecho(1);
@@ -1616,7 +1617,7 @@ String shell::path(path_t id)
 #else
         result = str(home) + "/.config/" + _domain;
 #endif
-        fsys::createDir(*result, 0700);
+        dir::create(*result, fsys::OWNER_PRIVATE);
         break;
     case USER_CACHE:
         home = ::getenv("HOME");
@@ -1647,7 +1648,7 @@ String shell::path(path_t id)
 #else
         result = str(home) + "/.config/" + _domain;
 #endif
-        fsys::createDir(*result, 0700);
+        dir::create(*result, fsys::OWNER_PRIVATE);
 
 #ifdef  __APPLE__
         result = result + "/" + _domain + ".conf";
@@ -1686,7 +1687,7 @@ String shell::path(path_t id)
     return result;
 }
 
-const char *shell::getenv(const char *id, const char *value)
+const char *shell::env(const char *id, const char *value)
 {
     const char *v = ::getenv(id);
     if(v)
@@ -1784,9 +1785,11 @@ static void exit_handler(void)
     }
 }
 
-static void abort_handler(int signo)
-{
-    exit_handler();
+extern "C" {
+    static void abort_handler(int signo)
+    {
+        exit_handler();
+    }
 }
 
 void shell::exiting(exitproc_t handler)
@@ -2448,7 +2451,7 @@ void shell::restart(char *argv0, char **argv, char **list)
 
 #endif
 
-bool shell::issym(const char *name)
+bool shell::is_sym(const char *name)
 {
     symlock.acquire();
 
@@ -2465,7 +2468,7 @@ bool shell::issym(const char *name)
     return false;
 }
 
-const char *shell::getsym(const char *name, const char *value)
+const char *shell::get(const char *name, const char *value)
 {
     symlock.acquire();
     linked_pointer<syms> sp = _syms;
@@ -2480,10 +2483,10 @@ const char *shell::getsym(const char *name, const char *value)
     }
 
     symlock.release();
-    return getenv(name, value);
+    return env(name, value);
 }
 
-void shell::setsym(const char *name, const char *value)
+void shell::set(const char *name, const char *value)
 {
     symlock.acquire();
     linked_pointer<syms> sp;
