@@ -88,43 +88,31 @@ public:
  * A thread-safe buffer for serializing and streaming class data.  While
  * the queue and stack operate by managing lists of reference pointers to
  * objects of various mixed kind, the buffer holds physical copies of objects
- * that being passed through it, and all must be the same size.  The buffer
- * class can be used stand-alone or with the typed bufferof template.  The
- * buffer is accessed in fifo order.
+ * that being passed through it, and all must be the same size.  For this
+ * reason the buffer is normally used through the bufferof<type> template
+ * rather than stand-alone.  The buffer is accessed in fifo order.
  * @author David Sugar <dyfet@gnutelephony.org>
  */
 class __EXPORT Buffer : protected Conditional
 {
 private:
-    size_t size, objsize;
+    size_t bufsize, objsize;
     caddr_t buf, head, tail;
-    unsigned count, limit;
+    unsigned objcount, limit;
 
-public:
+protected:
     /**
      * Create a buffer to hold a series of objects.
      * @param size of each object in buffer.
      * @param count of objects in the buffer.
      */
-    Buffer(size_t size, size_t count);
+    Buffer(size_t typesize, size_t count);
 
     /**
      * Deallocate buffer and unblock any waiting threads.
      */
     virtual ~Buffer();
 
-    /**
-     * Get the size of the buffer.
-     * @return size of the buffer.
-     */
-    unsigned getSize(void);
-
-    /**
-     * Get the number of objects in the buffer currently.
-     * @return number of objects buffered.
-     */
-    unsigned getCount(void);
-
     /**
      * Get the next object from the buffer.
      * @param timeout to wait when buffer is empty in milliseconds.
@@ -186,6 +174,21 @@ public:
      */
     void *peek(unsigned item);
 
+    virtual void *invalid(void) const;
+
+public:
+    /**
+     * Get the size of the buffer.
+     * @return size of the buffer.
+     */
+    unsigned size(void);
+
+    /**
+     * Get the number of objects in the buffer currently.
+     * @return number of objects buffered.
+     */
+    unsigned count(void);
+
     /**
      * Test if there is data waiting in the buffer.
      * @return true if buffer has data.
@@ -209,7 +212,7 @@ public:
  * slow.  If you need primarily lifo, you should use stack instead.
  * @author David Sugar <dyfet@gnutelephony.org>
  */
-class __EXPORT queue : protected OrderedIndex, protected Conditional
+class __EXPORT Queue : protected OrderedIndex, protected Conditional
 {
 private:
     mempager *pager;
@@ -219,7 +222,7 @@ private:
     class __LOCAL member : public OrderedObject
     {
     public:
-        member(queue *q, ObjectProtocol *obj);
+        member(Queue *q, ObjectProtocol *obj);
         ObjectProtocol *object;
     };
 
@@ -228,6 +231,8 @@ private:
 protected:
     size_t limit;
 
+    virtual ObjectProtocol *invalid(void) const;
+
 public:
     /**
      * Create a queue that uses a memory pager for internally managed
@@ -236,12 +241,12 @@ public:
      * @param number of pointers that can be in the queue or 0 for unlimited.
      * size limit.
      */
-    queue(mempager *pager = NULL, size_t number = 0);
+    Queue(mempager *pager = NULL, size_t number = 0);
 
     /**
      * Destroy queue.  If no mempager is used, then frees heap.
      */
-    ~queue();
+    ~Queue();
 
     /**
      * Remove a specific object pointer for the queue.  This can remove
@@ -267,7 +272,7 @@ public:
      * @param number of elements back.
      * @return object in queue or NULL if invalid value.
      */
-    ObjectProtocol *peek(unsigned offset = 0);
+    ObjectProtocol *get(unsigned offset = 0);
 
     /**
      * Get and remove last object posted to the queue.  This can wait for
@@ -291,55 +296,7 @@ public:
      * Get number of object points currently in the queue.
      * @return number of objects in queue.
      */
-    size_t getCount(void);
-
-    /**
-     * Convenience function to remove an object from the queue.
-     * @param queue to remove object from.
-     * @param object to remove.
-     * @return true if removed, false if not found.
-     */
-    static bool remove(queue& queue, ObjectProtocol *object)
-        {return queue.remove(object);};
-
-    /**
-     * Convenience function to post object into the queue.
-     * @param queue to post into.
-     * @param object to post.
-     * @param timeout to wait if full.
-     * @return true if posted, false if timed out while full.
-     */
-    static bool post(queue& queue, ObjectProtocol *object, timeout_t timeout = 0)
-        {return queue.post(object, timeout);};
-
-    /**
-     * Convenience function get first object from the queue.
-     * @param queue to get from.
-     * @param timeout to wait if empty.
-     * @return first object or NULL if timed out empty.
-     */
-    static ObjectProtocol *fifo(queue& queue, timeout_t timeout = 0)
-        {return queue.fifo(timeout);};
-
-    /**
-     * Convenience function get last object from the queue.
-     * @param queue to get from.
-     * @param timeout to wait if empty.
-     * @return last object or NULL if timed out empty.
-     */
-    static ObjectProtocol *lifo(queue& queue, timeout_t timeout = 0)
-        {return queue.lifo(timeout);};
-
-    /**
-     * Convenience function to get count of objects in the queue.
-     * @param queue to count.
-     * @return number of objects in the queue.
-     */
-    static size_t count(queue& queue)
-        {return queue.getCount();};
-
-    inline ObjectProtocol *operator[](unsigned pos)
-        {return peek(pos);};
+    size_t count(void);
 };
 
 /**
@@ -350,7 +307,7 @@ public:
  * member objects are used to operate the stack.
  * @author David Sugar <dyfet@gnutelephony.org>
  */
-class __EXPORT stack : protected Conditional
+class __EXPORT Stack : protected Conditional
 {
 private:
     LinkedObject *freelist, *usedlist;
@@ -360,7 +317,7 @@ private:
     class __LOCAL member : public LinkedObject
     {
     public:
-        member(stack *s, ObjectProtocol *obj);
+        member(Stack *s, ObjectProtocol *obj);
         ObjectProtocol *object;
     };
 
@@ -369,6 +326,8 @@ private:
 protected:
     size_t limit;
 
+    virtual ObjectProtocol *invalid(void) const;
+
 public:
     /**
      * Create a stack that uses a memory pager for internally managed
@@ -376,12 +335,12 @@ public:
      * @param pager to use for internal member object or NULL to use heap.
      * @param number of pointers that can be in the stack or 0 if unlimited.
      */
-    stack(mempager *pager = NULL, size_t number = 0);
+    Stack(mempager *pager = NULL, size_t number = 0);
 
     /**
      * Destroy queue.  If no pager is used, then frees heap.
      */
-    ~stack();
+    virtual ~Stack();
 
     /**
      * Remove a specific object pointer for the queue.  This can remove
@@ -416,52 +375,15 @@ public:
      * @param offset to stack entry.
      * @return object examined.
      */
-    ObjectProtocol *peek(unsigned offset = 0);
+    ObjectProtocol *get(unsigned offset = 0);
 
     /**
      * Get number of object points currently in the stack.
      * @return number of objects in stack.
      */
-    size_t getCount(void);
+    size_t count(void);
 
-    /**
-     * Convenience function to remove an object from the stacl.
-     * @param stack to remove object from.
-     * @param object to remove.
-     * @return true if removed, false if not found.
-     */
-    static inline bool remove(stack& stack, ObjectProtocol *object)
-        {return stack.remove(object);};
-
-    /**
-     * Convenience function to push object into the stack.
-     * @param stack to push into.
-     * @param object to push.
-     * @param timeout to wait if full.
-     * @return true if pusheded, false if timed out while full.
-     */
-    static inline bool push(stack& stack, ObjectProtocol *object, timeout_t timeout = 0)
-        {return stack.push(object, timeout);};
-
-    /**
-     * Convenience function pull last object from the stack.
-     * @param stack to get from.
-     * @param timeout to wait if empty.
-     * @return last object or NULL if timed out empty.
-     */
-    static inline ObjectProtocol *pull(stack& stack, timeout_t timeout = 0)
-        {return stack.pull(timeout);};
-
-    /**
-     * Convenience function to get count of objects in the stack.
-     * @param stack to count.
-     * @return number of objects in the stack.
-     */
-    static inline size_t count(stack& stack)
-        {return stack.getCount();};
-
-    inline ObjectProtocol *operator[](unsigned pos)
-        {return peek(pos);};
+    const ObjectProtocol *peek(timeout_t timeout = 0);
 };
 
 /**
@@ -516,8 +438,8 @@ public:
      * Create a buffer to hold a series of typed objects.
      * @param count of typed objects in the buffer.
      */
-    inline bufferof(unsigned count) :
-        Buffer(sizeof(T), count) {};
+    inline bufferof(unsigned capacity) :
+        Buffer(sizeof(T), capacity) {};
 
     /**
      * Get the next typed object from the buffer.  This blocks until an object
@@ -575,8 +497,8 @@ public:
      * @param item in buffer.
      * @return item pointer if valid or NULL.
      */
-    inline T *peek(unsigned item)
-        {return static_cast<T *>(Buffer::peek(item));};
+    inline const T& at(unsigned item)
+        {return static_cast<const T&>(Buffer::peek(item));};
 
     /**
      * Examine past item in the buffer.  This is a typecast of the peek
@@ -584,9 +506,11 @@ public:
      * @param item in buffer.
      * @return item pointer if valid or NULL.
      */
-    inline T *operator[](unsigned item)
-        {return static_cast<T *>(queue::peek(item));};
+    inline T&operator[](unsigned item)
+        {return static_cast<T&>(Buffer::peek(item));};
 
+    inline T* operator()(unsigned offset = 0)
+        {return static_cast<T*>(Buffer::peek(offset));}
 };
 
 /**
@@ -597,7 +521,7 @@ public:
  * @author David Sugar <dyfet@gnutelephony.org>
  */
 template<class T>
-class stackof : public stack
+class stackof : public Stack
 {
 public:
     /**
@@ -605,7 +529,7 @@ public:
      * @param memory pool for internal use of stack.
      * @param size of stack to construct.  Uses 0 if no size limit.
      */
-    inline stackof(mempager *memory, size_t size = 0) : stack(memory, size) {};
+    inline stackof(mempager *memory, size_t size = 0) : Stack(memory, size) {};
 
     /**
      * Remove a specific typed object pointer for the stack.  This can remove
@@ -615,7 +539,7 @@ public:
      * @return true if object was removed, false if not found.
      */
     inline bool remove(T *object)
-        {return stack::remove(object);};
+        {return Stack::remove(object);};
 
     /**
      * Push a typed object into the stack by it's pointer.  This can wait for
@@ -626,7 +550,7 @@ public:
      * @return true if object pushed, false if queue full and timeout expired.
      */
     inline bool push(T *object, timeout_t timeout = 0)
-        {return stack::push(object);};
+        {return Stack::push(object);};
 
     /**
      * Get and remove last typed object posted to the stack.  This can wait for
@@ -636,7 +560,19 @@ public:
      * @return object from queue or NULL if empty and timed out.
      */
     inline T *pull(timeout_t timeout = 0)
-        {return static_cast<T *>(stack::pull(timeout));};
+        {return static_cast<T *>(Stack::pull(timeout));};
+
+    /**
+     * Examine last typed object posted to the stack.  This can wait for
+     * a specified timeout of the stack is empty.
+     * @param timeout to wait if empty in milliseconds.
+     * @return object in queue or NULL if empty and timed out.
+     */
+    inline const T *peek(timeout_t timeout = 0)
+        {return static_cast<const T *>(Stack::peek(timeout));};
+
+    inline T* operator()(unsigned offset = 0)
+        {return static_cast<T*>(Stack::get(offset));}
 
     /**
      * Examine past item in the stack.  This is a typecast of the peek
@@ -644,8 +580,8 @@ public:
      * @param offset in stack.
      * @return item pointer if valid or NULL.
      */
-    inline T *peek(unsigned offset = 0)
-        {return static_cast<T *>(stack::peek(offset));};
+    inline const T& at(unsigned offset = 0)
+        {return static_cast<const T&>(Stack::get(offset));};
 
     /**
      * Examine past item in the stack.  This is a typecast of the peek
@@ -653,8 +589,8 @@ public:
      * @param offset in stack.
      * @return item pointer if valid or NULL.
      */
-    inline T *operator[](unsigned offset)
-        {return static_cast<T *>(stack::peek(offset));};
+    inline const T& operator[](unsigned offset)
+        {return static_cast<T&>(Stack::get(offset));};
 
 };
 
@@ -666,7 +602,7 @@ public:
  * @author David Sugar <dyfet@gnutelephony.org>
  */
 template<class T>
-class queueof : public queue
+class queueof : public Queue
 {
 public:
     /**
@@ -674,7 +610,7 @@ public:
      * @param memory pool for internal use by queue.
      * @param size of queue to construct.  Uses 0 if no size limit.
      */
-    inline queueof(mempager *memory, size_t size = 0) : queue(memory, size) {};
+    inline queueof(mempager *memory, size_t size = 0) : Queue(memory, size) {};
 
     /**
      * Remove a specific typed object pointer for the queue.  This can remove
@@ -684,7 +620,7 @@ public:
      * @return true if object was removed, false if not found.
      */
     inline bool remove(T *object)
-        {return queue::remove(object);};
+        {return Queue::remove(object);};
 
     /**
      * Post a typed object into the queue by it's pointer.  This can wait for
@@ -695,7 +631,7 @@ public:
      * @return true if object posted, false if queue full and timeout expired.
      */
     inline bool post(T *object, timeout_t timeout = 0)
-        {return queue::post(object);};
+        {return Queue::post(object);};
 
     /**
      * Get and remove first typed object posted to the queue.  This can wait for
@@ -705,7 +641,7 @@ public:
      * @return object from queue or NULL if empty and timed out.
      */
     inline T *fifo(timeout_t timeout = 0)
-        {return static_cast<T *>(queue::fifo(timeout));};
+        {return static_cast<T *>(Queue::fifo(timeout));};
 
     /**
      * Get and remove last typed object posted to the queue.  This can wait for
@@ -715,7 +651,7 @@ public:
      * @return object from queue or NULL if empty and timed out.
      */
     inline T *lifo(timeout_t timeout = 0)
-        {return static_cast<T *>(queue::lifo(timeout));};
+        {return static_cast<T *>(Queue::lifo(timeout));};
 
     /**
      * Examine past item in the queue.  This is a typecast of the peek
@@ -723,8 +659,8 @@ public:
      * @param offset in queue.
      * @return item pointer if valid or NULL.
      */
-    inline T *peek(unsigned offset = 0)
-        {return static_cast<T *>(queue::peek(offset));};
+    inline const T& at(unsigned offset = 0)
+        {return static_cast<const T&>(Queue::get(offset));};
 
     /**
      * Examine past item in the queue.  This is a typecast of the peek
@@ -732,70 +668,22 @@ public:
      * @param offset in queue.
      * @return item pointer if valid or NULL.
      */
-    inline T *operator[](unsigned offset)
-        {return static_cast<T *>(queue::peek(offset));};
+    inline T& operator[](unsigned offset)
+        {return static_cast<T&>(Queue::get(offset));};
 
+    inline T* operator()(unsigned offset = 0)
+        {return static_cast<T*>(Queue::get(offset));}
 };
 
 /**
  * Convenience type for using thread-safe object stacks.
  */
-typedef stack stack_t;
+typedef Stack stack_t;
 
 /**
  * Convenience type for using thread-safe object fifo (queue).
  */
-typedef queue fifo_t;
-
-/**
- * Convenience function to push an object onto a stack.
- * @param stack to push into.
- * @param object to push.
- */
-inline void push(stack_t &stack, ObjectProtocol *object)
-    {stack.push(object);}
-
-/**
- * Convenience function to pull an object from a stack.
- * @param stack to pull from.
- * @param timeout to wait to pull.
- * @return object pulled.
- */
-inline ObjectProtocol *pull(stack_t &stack, timeout_t timeout = Timer::inf)
-    {return stack.pull(timeout);}
-
-/**
- * Convenience function to remove an object from a stack.
- * @param stack to remove from.
- * @param object to remove.
- */
-inline void remove(stack_t &stack, ObjectProtocol *object)
-    {stack.remove(object);}
-
-/**
- * Convenience function to push an object onto a fifo.
- * @param fifo to push into.
- * @param object to push.
- */
-inline void push(fifo_t &fifo, ObjectProtocol *object)
-    {fifo.post(object);}
-
-/**
- * Convenience function to pull an object from a fifo.
- * @param fifo to pull from.
- * @param timeout to wait to pull.
- * @return object pulled.
- */
-inline ObjectProtocol *pull(fifo_t &fifo, timeout_t timeout = Timer::inf)
-    {return fifo.fifo(timeout);}
-
-/**
- * Convenience function to remove an object from a fifo.
- * @param fifo to remove from.
- * @param object to remove.
- */
-inline void remove(fifo_t &fifo, ObjectProtocol *object)
-    {fifo.remove(object);}
+typedef Queue fifo_t;
 
 END_NAMESPACE
 
