@@ -24,8 +24,8 @@
  * @file ucommon/fsys.h
  */
 
-#ifndef _UCOMMON_FILE_H_
-#define _UCOMMON_FILE_H_
+#ifndef _UCOMMON_FSYS_H_
+#define _UCOMMON_FSYS_H_
 
 #ifndef _UCOMMON_CONFIG_H_
 #include <ucommon/platform.h>
@@ -108,11 +108,6 @@
 
 NAMESPACE_UCOMMON
 
-/**
- * Convenience type for directory scan operations.
- */
-typedef void *dir_t;
-
 /**
  * Convenience type for loader operations.
  */
@@ -130,15 +125,24 @@ class __EXPORT fsys
 {
 protected:
     fd_t    fd;
-#ifdef  _MSWINDOWS_
-    WIN32_FIND_DATA *ptr;
-    HINSTANCE   mem;
-#else
-    void    *ptr;
-#endif
     int     error;
 
 public:
+    /**
+     * Most of the common chmod values are predefined.
+     */
+    enum {
+        OWNER_READONLY = 0400,
+        GROUP_READONLY = 0440,
+        PUBLIC_READONLY = 0444,
+        OWNER_PRIVATE = 0600,
+        OWNER_PUBLIC = 0644,
+        GROUP_PRIVATE = 0660,
+        GROUP_PUBLIC = 0664,
+        EVERYONE = 0666,
+        DIR_TEMPORARY = 01777
+    };
+
     typedef struct stat fileinfo_t;
 
 #ifdef  _MSWINDOWS_
@@ -152,15 +156,16 @@ public:
      * Enumerated file access modes.
      */
     typedef enum {
-        ACCESS_RDONLY,
-        ACCESS_WRONLY,
-        ACCESS_REWRITE,
-        ACCESS_RDWR = ACCESS_REWRITE,
-        ACCESS_APPEND,
-        ACCESS_SHARED,
-        ACCESS_DIRECTORY,
-        ACCESS_STREAM,
-        ACCESS_RANDOM
+        RDONLY,
+        WRONLY,
+        REWRITE,
+        RDWR = REWRITE,
+        APPEND,
+        SHARED,
+        EXCLUSIVE,
+        DEVICE,
+        STREAM,
+        RANDOM
     } access_t;
 
     /**
@@ -202,7 +207,7 @@ public:
      * @param access mode of file access.
      * @param permission mode of file.
      */
-    fsys(const char *path, access_t access, unsigned permission);
+    fsys(const char *path, unsigned permission, access_t access);
 
     /**
      * Close and release a file descriptor.
@@ -221,21 +226,21 @@ public:
      * @return low level file handle.
      */
     inline operator fd_t() const
-        {return fd;};
+        {return fd;}
 
     /**
      * Test if file descriptor is open.
      * @return true if open.
      */
     inline operator bool() const
-        {return fd != INVALID_HANDLE_VALUE || ptr != NULL;};
+        {return fd != INVALID_HANDLE_VALUE;}
 
     /**
      * Test if file descriptor is closed.
      * @return true if closed.
      */
     inline bool operator!() const
-        {return fd == INVALID_HANDLE_VALUE && ptr == NULL;};
+        {return fd == INVALID_HANDLE_VALUE;}
 
     /**
      * Assign file descriptor by duplicating another descriptor.
@@ -260,7 +265,7 @@ public:
      * Get the native system descriptor handle of the file descriptor.
      * @return native os descriptor.
      */
-    inline fd_t getHandle(void) const
+    inline fd_t handle(void) const
         {return fd;};
 
     /**
@@ -293,13 +298,13 @@ public:
      * See if current file stream is a tty device.
      * @return true if device.
      */
-    bool istty(void);
+    bool is_tty(void);
 
     /**
      * See if the file handle is a tty device.
      * @return true if device.
      */
-    static bool istty(fd_t fd);
+    static bool is_tty(fd_t fd);
 
     /**
      * Read data from descriptor or scan directory.
@@ -322,7 +327,7 @@ public:
      * @param buffer to save status info in.
      * @return error number or 0 on success.
      */
-    int fileinfo(fileinfo_t *buffer);
+    int info(fileinfo_t *buffer);
 
     /**
      * Truncate file to specified length.  The file pointer is positioned
@@ -343,7 +348,7 @@ public:
      * @param path to change to.
      * @return error number or 0 on success.
      */
-    static int changeDir(const char *path);
+    static int prefix(const char *path);
 
     /**
      * Get current directory prefix (pwd).
@@ -351,7 +356,9 @@ public:
      * @param size of path we can save.
      * @return error number or 0 on success.
      */
-    static int getPrefix(char *path, size_t size);
+    static int prefix(char *path, size_t size);
+
+    static string_t prefix(void);
 
     /**
      * Stat a file.
@@ -359,14 +366,14 @@ public:
      * @param buffer to save stat info.
      * @return error number or 0 on success.
      */
-    static int fileinfo(const char *path, fileinfo_t *buffer);
+    static int info(const char *path, fileinfo_t *buffer);
 
     /**
-     * Remove a file.
+     * Erase (remove) a file only.
      * @param path of file.
      * @return error number or 0 on success.
      */
-    static int remove(const char *path);
+    static int erase(const char *path);
 
     /**
      * Copy a file.
@@ -388,84 +395,73 @@ public:
     /**
      * Change file access mode.
      * @param path to change.
-     * @param mode to assign.
+     * @param value of mode to assign.
      * @return error number or 0 on success.
      */
-    static int change(const char *path, unsigned mode);
+    static int mode(const char *path, unsigned value);
 
     /**
-     * Test path access.
+     * Test if path exists.
      * @param path to test.
-     * @param mode to test for.
-     * @return error number or 0 on success.
+     * @return if true.
      */
-    static int access(const char *path, unsigned mode);
+    static bool is_exists(const char *path);
+
+    /**
+     * Test if path readable.
+     * @param path to test.
+     * @return if true.
+     */
+    static bool is_readable(const char *path);
+
+    /**
+     * Test if path writable.
+     * @param path to test.
+     * @return if true.
+     */
+    static bool is_writable(const char *path);
+
+    /**
+     * Test if path is executable.
+     * @param path to test.
+     * @return if true.
+     */
+    static bool is_executable(const char *path);
 
     /**
      * Test if path is a file.
      * @param path to test.
      * @return true if exists and is file.
      */
-    static bool isfile(const char *path);
+    static bool is_file(const char *path);
 
     /**
      * Test if path is a directory.
      * @param path to test.
      * @return true if exists and is directory.
      */
-    static bool isdir(const char *path);
+    static bool is_dir(const char *path);
 
     /**
      * Test if path is a symlink.
      * @param path to test.
      * @return true if exists and is symlink.
      */
-    static bool islink(const char *path);
+    static bool is_link(const char *path);
+
+    /**
+     * Test if path is a device path.
+     * @param path to test.
+     * @return true of is a device path.
+     */
+    static bool is_device(const char *path);
 
     /**
      * Test if path is a hidden file.
      * @param path to test.
      * @return true if exists and is hidden.
      */
-    static bool ishidden(const char *path);
-
-    /**
-     * Read data from file descriptor or directory.
-     * @param descriptor to read from.
-     * @param buffer to read into.
-     * @param count of bytes to read.
-     * @return bytes transferred, -1 if error.
-     */
-    inline static ssize_t read(fsys& descriptor, void *buffer, size_t count)
-        {return descriptor.read(buffer, count);};
-
-    /**
-     * write data to file descriptor.
-     * @param descriptor to write to.
-     * @param buffer to write from.
-     * @param count of bytes to write.
-     * @return bytes transferred, -1 if error.
-     */
-    inline static ssize_t write(fsys& descriptor, const void *buffer, size_t count)
-        {return descriptor.write(buffer, count);};
-
-    /**
-     * Set the position of a file descriptor.
-     * @param descriptor to set.
-     * @param offset from start of file or "end" to append.
-     * @return error number or 0 on success.
-     */
-    inline static int seek(fsys& descriptor, offset_t offset)
-        {return descriptor.seek(offset);};
-
-    /**
-     * Drop cached data from a file descriptor.
-     * @param descriptor to set.
-     * @param size of region from start of file to drop or all.
-     * @return error number or 0 on success.
-     */
-    inline static int drop(fsys& descriptor, offset_t size)
-        {return descriptor.drop(size);};
+    static bool is_hidden(const char *path);
 
     /**
      * Open a file or directory.
@@ -495,22 +491,7 @@ public:
      * @param access mode of descriptor.
      * @param mode of file if created.
      */
-    void create(const char *path, access_t access, unsigned mode);
-
-    /**
-     * Simple direct method to create a directory.
-     * @param path of directory to create.
-     * @param mode of directory.
-     * @return error number or 0 on success.
-     */
-    static int createDir(const char *path, unsigned mode);
-
-    /**
-     * Simple direct method to remove a directory.
-     * @param path to remove.
-     * @return error number or 0 on success.
-     */
-    static int removeDir(const char *path);
+    void open(const char *path, unsigned mode, access_t access);
 
     /**
      * Remove a symbolic link explicitly.  Other kinds of files are also
@@ -545,17 +526,11 @@ public:
      */
     static int linkinfo(const char *path, char *buffer, size_t size);
 
-    /**
-     * Close a file descriptor or directory directly.
-     * @param descriptor to close.
-     */
-    inline static void close(fsys& descriptor)
-        {descriptor.close();};
-
     /**
      * Close a fsys resource.
+     * @return error code as needed.
      */
-    void close(void);
+    int close(void);
 
     /**
      * Get last error.
@@ -564,15 +539,6 @@ public:
     inline int err(void) const
         {return error;}
 
-    /**
-     * Open a file or directory.
-     * @param object to assign descriptor for opened file.
-     * @param path of file to open.
-     * @param access mode of descriptor.
-     */
-    inline static void open(fsys& object, const char *path, access_t access)
-        {object.open(path, access);};
-
     /**
      * Direct means to open a read-only file path and return a descriptor.
      * @param path to open.
@@ -623,22 +589,12 @@ public:
      * Create inheritable /dev/null handle.
      * @return null device handle.
      */
-    static fd_t nullfile(void);
+    static fd_t null(void);
 
     /**
-     * create a file descriptor or directory directly.
-     * @param object to assign descriptor for created file.
-     * @param path of file to create.
-     * @param access mode of descriptor.
-     * @param mode of file if created.
-     */
-    inline static void create(fsys& object, const char *path, access_t access, unsigned mode)
-        {object.create(path, access, mode);};
-
-    /**
-     * Load an unmaged plugin directly.
+     * Load a library into memory.
      * @param path to plugin.
-     * @return error number or 0 on success.
+     * @return 0 on success, else error.
      */
     static int load(const char *path);
 
@@ -646,24 +602,82 @@ public:
      * Execute a process and get exit code.
      * @param path to execute.
      * @param argv list.
+     * @param optional env.
      * @return exit code.
      */
-    static int exec(const char *path, char **argv);
+    static int exec(const char *path, char **argv, char **envp = NULL);
 
-    static int exec(const char *path, char **argv, char **envp);
+    static inline bool is_file(struct stat *inode)
+        {return S_ISREG(inode->st_mode);}
+
+    static inline bool is_dir(struct stat *inode)
+        {return S_ISDIR(inode->st_mode);}
+
+    static inline bool is_link(struct stat *inode)
+        {return S_ISLNK(inode->st_mode);}
+
+    static inline bool is_dev(struct stat *inode)
+        {return S_ISBLK(inode->st_mode) || S_ISCHR(inode->st_mode);}
+
+    static inline bool is_char(struct stat *inode)
+        {return S_ISCHR(inode->st_mode);}
+
+    static inline bool is_disk(struct stat *inode)
+        {return S_ISBLK(inode->st_mode);}
+
+    static inline bool is_sys(struct stat *inode)
+        {return S_ISSOCK(inode->st_mode) || S_ISFIFO(inode->st_mode);}
+};
+
+/**
+ * Convenience class for library plugins.
+ * @author David Sugar <dyfet@gnutelephony.org>
+ */
+class __EXPORT dso
+{
+private:
+    friend class fsys;
+
+#ifdef  _MSWINDOWS_
+    HINSTANCE   ptr;
+#else
+    void    *ptr;
+#endif
+    int     error;
+
+public:
+#ifdef  _MSWINDOWS_
+    typedef int (FAR WINAPI *addr_t)();
+#else
+    typedef void *addr_t;
+#endif
 
     /**
-     * Load a plugin into memory.
-     * @param module for management.
-     * @param path to plugin.
+     * Create dso object for use by load functions.
      */
-    static void load(fsys& module, const char *path);
+    dso();
 
     /**
-     * unload a specific plugin.
-     * @param module to unload
+     * Create and map a dso object.
+     * @param path of library to map.
      */
-    static void unload(fsys& module);
+    dso(const char *path);
+
+    /**
+     * Destroy dso and release library.
+     */
+    ~dso();
+
+    /**
+     * Map library object with library.
+     * @param name of library to load.
+     */
+    void map(const char *path);
+
+    /**
+     * Release loaded library.
+     */
+    void release(void);
 
     /**
      * Find symbol in loaded module.
@@ -671,170 +685,139 @@ public:
      * @param symbol to search for.
      * @return address of symbol or NULL if not found.
      */
-    static void *find(fsys& module, const char *symbol);
+    addr_t find(const char *symbol) const;
 
-    static inline bool isfile(struct stat *inode)
-        {return S_ISREG(inode->st_mode);}
+    inline int err(void) const
+        {return error;}
 
-    static inline bool isdir(struct stat *inode)
-        {return S_ISDIR(inode->st_mode);}
+    inline addr_t operator[](const char *symbol) const
+        {return find(symbol);}
 
-    static inline bool islink(struct stat *inode)
-        {return S_ISLNK(inode->st_mode);}
+    inline addr_t operator()(const char *symbol) const
+        {return find(symbol);}
 
-    static inline bool isdev(struct stat *inode)
-        {return S_ISBLK(inode->st_mode) || S_ISCHR(inode->st_mode);}
+    inline operator bool()
+        {return ptr != NULL;}
 
-    static inline bool ischar(struct stat *inode)
-        {return S_ISCHR(inode->st_mode);}
-
-    static inline bool isdisk(struct stat *inode)
-        {return S_ISBLK(inode->st_mode);}
-
-    static inline bool issys(struct stat *inode)
-        {return S_ISSOCK(inode->st_mode) || S_ISFIFO(inode->st_mode);}
+    inline bool operator!()
+        {return ptr == NULL;}
 };
 
 /**
- * Access standard files through character protocol.  This can also be
- * used as an alternative means to access files that manages file pointers.
+ * Convenience class for directories.
  * @author David Sugar <dyfet@gnutelephony.org>
  */
-class __EXPORT charfile : public CharacterProtocol
+class __EXPORT dir : private fsys
 {
 private:
-    FILE *fp;
-    bool opened;
-
-    int _putch(int code);
-
-    int _getch(void);
+#ifdef  _MSWINDOWS_
+    WIN32_FIND_DATA *ptr;
+    HINSTANCE   mem;
+#else
+    void    *ptr;
+#endif
 
 public:
-    typedef ::fpos_t bookmark_t;
-
     /**
-     * Construct a charfile from an existing FILE pointer.
-     * @param file to use.
+     * Construct and open a directory path.
+     * @param path of directory.
      */
-    inline charfile(FILE *file)
-        {fp = file; opened = false;}
+    dir(const char *path);
 
     /**
-     * Construct an open charfile based on a path and mode.
-     * @param path of file to open.
-     * @param mode of file.
+     * Construct an unopened directory.
      */
-    charfile(const char *path, const char *mode);
+    dir();
 
     /**
-     * Construct an unopened file.
+     * Close and release directory.
      */
-    charfile();
+    ~dir();
 
     /**
-     * Destroy object and close associated file.
+     * Simple direct method to create a directory.
+     * @param path of directory to create.
+     * @param mode of directory.
+     * @return error number or 0 on success.
      */
-    ~charfile();
+    static int create(const char *path, unsigned mode);
 
     /**
-     * Test if file is opened.
-     * @return true if opened.
+     * Remove an empty directory.
+     * @param path of directory.
+     * @return error number or 0 on success.
      */
-    inline operator bool()
-        {return fp != NULL;}
+    static int remove(const char *path);
 
     /**
-     * Test if file is not opened.
-     * @return true if not opened.
+     * Open a directory path for reading.
+     * @param path to open.
      */
-    inline bool operator !()
-        {return fp == NULL;}
-
-    inline operator FILE *()
-        {return fp;}
+    void open(const char *path);
 
     /**
-     * Open file path.  If a file is already opened, it is closed.
-     * @param path of file to open.
-     * @param mode of file to open.
+     * Read data from directory.
+     * @param buffer to read into.
+     * @param count of bytes to read.
+     * @return bytes transferred, -1 if error.
      */
-    void open(const char *path, const char *mode);
+    ssize_t read(char *buffer, size_t count);
 
     /**
-     * Close an open file.
+     * Close and release directory object.
      */
     void close(void);
 
-    /**
-     * Put a string into the file.
-     * @param string to write.
-     * @return number of characters written.
-     */
-    size_t put(const char *string);
+    inline int err(void) const
+        {return fsys::err();}
 
     /**
-     * Read a line of input from the file.  This clears the newline
-     * character at the end and has consistent behavior with other
-     * ucommon file routines.  Because the newline is cleared, the
-     * string length may be shorter than the return size.
-     * @param string to write.
-     * @param size of buffer.
-     * @return true if data read, 0 if at end of file.
+     * Test if file descriptor is open.
+     * @return true if open.
      */
-    size_t readline(char *string, size_t size);
+    inline operator bool() const
+        {return ptr != NULL;};
 
     /**
-     * Read a string of input from the file.  This clears the newline
-     * character at the end and has consistent behavior with other
-     * ucommon file routines.  Because the newline is cleared, the
-     * string length may be shorter than the return size.
-     * @param string to write.
-     * @return true if data read, 0 if at end of file.
+     * Test if file descriptor is closed.
+     * @return true if closed.
      */
-    size_t readline(string& string);
-
-    inline size_t put(const void *data, size_t size)
-        { return fp == NULL ? 0 : fwrite(data, 1, size, fp);}
-
-    size_t get(void *data, size_t size)
-        { return fp == NULL ? 0 : fread(data, 1, size, fp);}
-
-    inline void get(bookmark_t& pos)
-        { if(fp) fsetpos(fp, &pos);}
-
-    inline void set(bookmark_t& pos)
-        { if(fp) fgetpos(fp, &pos);}
-
-    int err(void);
-
-    bool eof(void);
-
-    inline void seek(long offset)
-        {if(fp) fseek(fp, offset, SEEK_SET);}
-
-    inline void move(long offset)
-        {if(fp) fseek(fp, offset, SEEK_CUR);}
-
-    inline void append(void)
-        {if (fp) fseek(fp, 0l, SEEK_END);}
-
-    inline void rewind(void)
-        {if(fp) ::rewind(fp);}
-
-    size_t printf(const char *format, ...) __PRINTF(2, 3);
-
-    bool istty(void);
+    inline bool operator!() const
+        {return ptr == NULL;};
 };
 
-String str(charfile& fp, strsize_t size);
-
 /**
  * Convience type for fsys.
  */
 typedef fsys fsys_t;
 
-extern charfile cstdin, cstdout, cstderr;
+typedef dir dir_t;
+
+typedef dso dso_t;
+
+inline bool is_exists(const char *path)
+    {return fsys::is_exists(path);}
+
+inline bool is_readable(const char *path)
+    {return fsys::is_readable(path);}
+
+inline bool is_writable(const char *path)
+    {return fsys::is_writable(path);}
+
+inline bool is_executable(const char *path)
+    {return fsys::is_executable(path);}
+
+inline bool is_file(const char *path)
+    {return fsys::is_file(path);}
+
+inline bool is_dir(const char *path)
+    {return fsys::is_dir(path);}
+
+inline bool is_link(const char *path)
+    {return fsys::is_link(path);}
+
+inline bool is_device(const char *path)
+    {return fsys::is_device(path);}
 
 END_NAMESPACE
 
