@@ -59,7 +59,7 @@ protected:
     friend class NamedObject;
     friend class ObjectStack;
 
-    LinkedObject *next;
+    LinkedObject *Next;
 
     /**
      * Construct base class attached to a chain of objects.
@@ -110,7 +110,7 @@ public:
      * Search to see if we are a member of a specific list.
      * @return true if we are member of the list.
      */
-    bool isMember(LinkedObject *list) const;
+    bool is_member(LinkedObject *list) const;
 
     /**
      * Release all objects from a list.
@@ -137,7 +137,7 @@ public:
      * @return next linked object in list.
      */
     inline LinkedObject *getNext(void) const
-        {return next;};
+        {return Next;};
 };
 
 /**
@@ -182,12 +182,17 @@ protected:
 
     OrderedObject *head, *tail;
 
+    void copy(const OrderedIndex& source);
+
 public:
     /**
      * Create and initialize an empty index.
      */
     OrderedIndex();
 
+    inline OrderedIndex(const OrderedIndex& source)
+        {copy(source);}
+
     /**
      * Destroy index.
      */
@@ -277,6 +282,13 @@ public:
     inline LinkedObject *operator*() const
         {return (LinkedObject*)(head);};
 
+    /**
+     * Assign ordered index.
+     * @param object to copy from.
+     */
+    OrderedIndex& operator=(const OrderedIndex& object)
+        {copy(object); return *this;}
+
     /**
      * Add object to our list.
      * @param object to add.
@@ -364,7 +376,7 @@ protected:
     void delist(void);
 
 private:
-    DLinkedObject *prev;
+    DLinkedObject *Prev;
 };
 
 /**
@@ -384,7 +396,7 @@ private:
 class __EXPORT NamedObject : public OrderedObject
 {
 protected:
-    char *id;
+    char *Id;
 
     /**
      * Create an empty unnamed cell object.
@@ -527,7 +539,7 @@ public:
      * @return name id.
      */
     inline char *getId(void) const
-        {return id;};
+        {return Id;};
 
     /**
      * Compare the name of our object to see if equal.  This is a virtual
@@ -573,8 +585,8 @@ public:
 class __EXPORT NamedTree : public NamedObject
 {
 protected:
-    NamedTree *parent;
-    OrderedIndex child;
+    NamedTree *Parent;
+    OrderedIndex Child;
 
     /**
      * Create a stand-alone or root tree node, with an optional name.
@@ -663,14 +675,14 @@ public:
      * @return first child node or NULL if no children.
      */
     inline NamedTree *getFirst(void) const
-        {return static_cast<NamedTree *>(child.begin());};
+        {return static_cast<NamedTree *>(Child.begin());};
 
     /**
      * Get parent node we are listed as a child on.
      * @return parent node or NULL if none.
      */
     inline NamedTree *getParent(void) const
-        {return parent;};
+        {return Parent;};
 
     /**
      * Get child by index number.
@@ -678,28 +690,28 @@ public:
      * @return indexed child node.
      */
     inline NamedTree *getIndexed(unsigned index) const
-        {return static_cast<NamedTree *>(child.getIndexed(index));};
+        {return static_cast<NamedTree *>(Child.getIndexed(index));};
 
     /**
      * Get the ordered index of our child nodes.
      * @return ordered index of our children.
      */
     inline OrderedIndex *getIndex(void) const
-        {return const_cast<OrderedIndex*>(&child);};
+        {return const_cast<OrderedIndex*>(&Child);};
 
     /**
      * Test if this node has a name.
      * @return true if name is set.
      */
     inline operator bool() const
-        {return (id != NULL);};
+        {return (Id != NULL);};
 
     /**
      * Test if this node is unnamed.
      * @return false if name is set.
      */
     inline bool operator!() const
-        {return (id == NULL);};
+        {return (Id == NULL);};
 
     /**
      * Set or replace the name id of this node.  This will free the string
@@ -718,15 +730,15 @@ public:
      * Test if node has children.
      * @return true if node contains child nodes.
      */
-    inline bool isLeaf(void) const
-        {return (child.begin() == NULL);};
+    inline bool is_leaf(void) const
+        {return (Child.begin() == NULL);};
 
     /**
      * Test if node is root node.
      * @return true if node is root node.
      */
-    inline bool isRoot(void) const
-        {return (parent == NULL);};
+    inline bool is_root(void) const
+        {return (Parent == NULL);};
 
     /**
      * Add leaf to a trunk, by order.  If NULL, just remove.
@@ -759,8 +771,8 @@ class __EXPORT LinkedList : public OrderedObject
 protected:
     friend class ObjectQueue;
 
-    LinkedList *prev;
-    OrderedIndex *root;
+    LinkedList *Prev;
+    OrderedIndex *Root;
 
     /**
      * Construct and add our object to an existing double linked list at end.
@@ -810,22 +822,22 @@ public:
      * Test if we are at the head of a list.
      * @return true if we are the first node in a list.
      */
-    inline bool isHead(void) const
-        {return root->head == (OrderedObject *)this;};
+    inline bool is_head(void) const
+        {return Root->head == (OrderedObject *)this;};
 
     /**
      * Test if we are at the end of a list.
      * @return true if we are the last node in a list.
      */
-    inline bool isTail(void) const
-        {return root->tail == (OrderedObject *)this;};
+    inline bool is_tail(void) const
+        {return Root->tail == (OrderedObject *)this;};
 
     /**
      * Get previous node in the list for reverse iteration.
      * @return previous node in list.
      */
     inline LinkedList *getPrev(void) const
-        {return prev;};
+        {return Prev;};
 
     /**
      * Get next node in the list when iterating.
@@ -1300,6 +1312,9 @@ public:
     inline linked_pointer(LinkedObject *pointer)
         {ptr = static_cast<T*>(pointer);};
 
+    inline linked_pointer(const LinkedObject *pointer)
+        {ptr = static_cast<T*>(pointer);};
+
     /**
      * Create a linked pointer to examine an ordered index.
      * @param index of linked objects to iterate through.
@@ -1405,14 +1420,14 @@ public:
      * Test for next member in linked list.
      * @return true if there is more members after current one.
      */
-    inline bool isNext(void) const
+    inline bool is_next(void) const
         {return (ptr->getNext() != NULL);};
 
     /**
      * Test for previous member in double linked list.
      * @return true if there is more members before current one.
      */
-    inline bool isPrev(void) const
+    inline bool is_prev(void) const
         {return (ptr->getPrev() != NULL);};
 
     /**
@@ -1597,8 +1612,8 @@ public:
      * Test if this node is a leaf node for a tree pointer table.
      * @return true if value pointer is not NULL and there are no children.
      */
-    inline bool isAttribute(void) const
-        {return (!child.begin() && value != NULL);};
+    inline bool is_attribute(void) const
+        {return (!Child.begin() && value != NULL);};
 
     /**
      * Get the pointer of a pointer based value tree.
@@ -1641,14 +1656,14 @@ public:
      * @return node or NULL if past end.
      */
     inline treemap *getIndexed(unsigned index) const
-        {return static_cast<treemap*>(child.getIndexed(index));};
+        {return static_cast<treemap*>(Child.getIndexed(index));};
 
     /**
      * Get the typed parent node for our node.
      * @return parent node or NULL if root of tree.
      */
     inline treemap *getParent(void) const
-        {return static_cast<treemap*>(parent);};
+        {return static_cast<treemap*>(Parent);};
 
     /**
      * Get direct typed child node of our node of specified name.  This
@@ -1826,6 +1841,11 @@ public:
      */
     inline T **sort(void) const
         {return NamedObject::sort(NamedObject::index(idx, M));};
+
+    /**
+     * Convenience typedef for iterative pointer.
+     */
+    typedef linked_pointer<T> iterator;
 };
 
 /**
@@ -1843,7 +1863,7 @@ public:
      * @return pointer to index root.
      */
     inline NamedObject **root(void)
-        {return static_cast<NamedObject*>(&head);};
+        {return static_cast<NamedObject**>(&head);};
 
     /**
      * Return first item in ordered list.  This is commonly used to
@@ -1917,104 +1937,13 @@ public:
      */
     inline T **sort(void)
         {return static_cast<T**>(NamedObject::sort(index()));};
+
+    /**
+     * Convenience typedef for iterative pointer.
+     */
+    typedef linked_pointer<T> iterator;
 };
 
-/**
- * Push a linked object onto a stack of linked objects.
- * @param stack to push object onto.
- * @param object to push onto stack.
- */
-inline void push(ObjectStack& stack, LinkedObject *object)
-    {stack.push(object);}
-
-/**
- * Add a linked object onto a stack of linked objects.
- * @param stack to push object onto.
- * @param object to push onto stack.
- */
-inline void add(ObjectStack& stack, LinkedObject *object)
-    {stack.push(object);}
-
-/**
- * Pop a linked object from a stack of linked objects.
- * @param stack to pull object from.
- * @return object pulled from stack or NULL if none.
- */
-inline LinkedObject *pop(ObjectStack& stack)
-    {return stack.pull();}
-
-/**
- * Pop a linked object from a stack of linked objects.
- * @param stack to pull object from.
- * @return object pulled from stack or NULL if none.
- */
-inline LinkedObject *pull(ObjectStack& stack)
-    {return stack.pull();}
-
-/**
- * Push a linked object onto a fifo of linked objects.
- * @param fifo to push object onto.
- * @param object to push onto fifo.
- */
-inline void push(OrderedIndex& fifo, LinkedObject *object)
-    {fifo.add((OrderedObject *)object);}
-
-/**
- * Add a linked object onto a fifo of linked objects.
- * @param fifo to push object onto.
- * @param object to push onto fifo.
- */
-inline void add(OrderedIndex& fifo, LinkedObject *object)
-    {fifo.add((OrderedObject *)object);}
-
-/**
- * Pop a linked object from a fifo of linked objects.
- * @param fifo to pull object from.
- * @return object pulled from stack or NULL if none.
- */
-inline LinkedObject *pop(OrderedIndex& fifo)
-    {return fifo.get();}
-
-/**
- * Pop a linked object from a fifo of linked objects.
- * @param fifo to pull object from.
- * @return object pulled from stack or NULL if none.
- */
-inline LinkedObject *pull(OrderedIndex& fifo)
-    {return fifo.get();}
-
-/**
- * Push a linked object to start of queue.
- * @param queue to push object into.
- * @param object to push onto fifo.
- */
-inline void push(ObjectQueue& queue, DLinkedObject *object)
-    {queue.add(object);}
-
-/**
- * Add a linked object to end of queue of objects.
- * @param queue to add object onto.
- * @param object to add onto queue.
- */
-inline void add(ObjectQueue& queue, DLinkedObject *object)
-    {queue.add(object);}
-
-/**
- * Pop a linked object from end of queue.
- * @param queue to pop object from.
- * @return object popped from queue or NULL if none.
- */
-inline DLinkedObject *pop(ObjectQueue& queue)
-    {return (DLinkedObject *)queue.pop();}
-
-/**
- * Pull a linked object from start of a queue.
- * @param queue to pull object from.
- * @return object pulled from queue or NULL if none.
- */
-inline DLinkedObject *pull(ObjectQueue& queue)
-    {return (DLinkedObject *)queue.pull();}
-
 /**
  * Convenience typedef for root pointers of single linked lists.
  */
