@@ -26,7 +26,7 @@
 
 #ifndef _UCOMMON_PLATFORM_H_
 #define _UCOMMON_PLATFORM_H_
-#define UCOMMON_ABI 5
+#define UCOMMON_ABI 6
 
 /**
  * Common namespace for all ucommon objects.
@@ -313,15 +313,21 @@ typedef char *caddr_t;
 #include <sys/types.h>
 #include <stdint.h>
 #include <unistd.h>
+#include <stdio.h>
 
 #endif
 
+#undef  getchar
+#undef  putchar
+
 #ifndef _GNU_SOURCE
 typedef void (*sighandler_t)(int);  /**< Convenient typedef for signal handlers. */
 #endif
 typedef unsigned long timeout_t;    /**< Typedef for millisecond timer values. */
 
 #include <stdlib.h>
+#include <ctype.h>
+#include <limits.h>
 #include <errno.h>
 
 #ifdef  _MSWINDOWS_
