@@ -99,19 +99,19 @@ static void digest(const char *path = NULL)
     unsigned char buffer[1024];
 
     if(path) {
-        int err = fsys::fileinfo(path, &ino);
+        int err = fsys::info(path, &ino);
 
         if(err) {
             result(path, err);
             return;
         }
 
-        if(fsys::issys(&ino)) {
+        if(fsys::is_sys(&ino)) {
             result(path, EBADF);
             return;
         }
 
-        fs.open(path, fsys::ACCESS_STREAM);
+        fs.open(path, fsys::STREAM);
     }
     else
         fs.assign(shell::input());
@@ -136,10 +136,10 @@ static void digest(const char *path = NULL)
 static void scan(String path, bool top = true)
 {
     char filename[128];
-    String filepath;
-    fsys_t dir(path, fsys::ACCESS_DIRECTORY);
+    string_t filepath;
+    dir_t dir(path);
 
-    while(is(dir) && fsys::read(dir, filename, sizeof(filename))) {
+    while(is(dir) && dir.read(filename, sizeof(filename))) {
         if(*filename == '.' && (filename[1] == '.' || !filename[1]))
             continue;
 
@@ -147,7 +147,7 @@ static void scan(String path, bool top = true)
             continue;
 
         filepath = str(path) + str("/") + str(filename);
-        if(fsys::isdir(filepath)) {
+        if(fsys::is_dir(filepath)) {
             if(is(recursive) || is(altrecursive))
                 scan(filepath, false);
             else
@@ -177,20 +177,20 @@ PROGRAM_MAIN(argc, argv)
     }
 
     secure::init();
-    if(!Digest::is(*hash))
+    if(!Digest::has(*hash))
         shell::errexit(2, "*** %s: %s: %s\n",
             argv0, *hash, _TEXT("unkown or unsupported digest method"));
 
     md = *hash;
 
     // we can symlink md as md5, etc, to set alternate default digest names
-    if(!is(hash) && Digest::is(argv0))
+    if(!is(hash) && Digest::has(argv0))
         md = argv0;
 
     if(!args())
         digest();
     else while(count < args()) {
-        if(fsys::isdir(args[count]))
+        if(fsys::is_dir(args[count]))
             scan(str(args[count++]));
         else
             digest(args[count++]);
